<?php

namespace App\Data;

use App\Exceptions\ItemNotFoundException;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Str;
use App\Config\GiteaConfig;

class ItemsCollection
{
    protected Collection $items;
    protected Collection $locales;

    private string $items_cache_key = 'items';
    private string $locales_cache_key = 'locales';

    public function __construct() {
        if (!Cache::has($this->items_cache_key)) {
            $this->refreshItemsCache();
        }
        else  $this->items = Cache::get($this->items_cache_key);

        if (!Cache::has($this->locales_cache_key)) {
            $this->refreshLocalesCache();
        }
        else $this->locales = Cache::get($this->locales_cache_key);
    }

    public function refreshLocalesCache(): void
    {
        $this->locales = collect();
        $raw_locales_global_base_url = GiteaConfig::RAW_LOCALES_GLOBAL_BASE_URL;

        // Getting all locales in project/assets/database/locales/global from the Server development branch repository
        $locales_list = collect(json_decode(file_get_contents(GiteaConfig::LOCALES_GLOBAL_URL), true));
        foreach($locales_list as $item) {
            // Extract the json name for the locale
            preg_match('/([a-z]{2}(-[a-z]{2})?).json/', $item['name'], $current_locale_name, PREG_OFFSET_CAPTURE);

            // If the name is not supported for any reason, dont add it to the locales
            if (empty($current_locale_name) || !$current_locale_name[1][0]) continue;

            $trimmed_current_locale_name = trim($current_locale_name[1][0]);
            $current_locale_json = json_decode(file_get_contents("${raw_locales_global_base_url}/${trimmed_current_locale_name}.json"), true);
            $template_locale = collect($current_locale_json['templates']);
            $customization_locale= collect($current_locale_json['customization']);
            $this->locales = $this->locales->merge([$trimmed_current_locale_name => $template_locale->concat($customization_locale)]);
        }
        Cache::put($this->locales_cache_key, $this->locales);
    }

    public function refreshItemsCache(): void {
        $this->items = collect(json_decode(file_get_contents(GiteaConfig::RAW_ITEMS_URL), true));
        $this->items = $this->items->merge(collect(json_decode(file_get_contents(GiteaConfig::RAW_CUSTOMIZATION_URL), true)));
        Cache::put($this->items_cache_key, $this->items);
    }

    public function getLocales(): Collection {
        return $this->locales->keys();
    }

    public function refreshAllCache(): void {
        $this->refreshItemsCache();
        $this->refreshLocalesCache();
    }

    /**
     * Checks if the query is in the key
     */
    private function contains(string $key, string $query) : bool {
        $key = Str::lower(trim($key));
        $query = Str::lower(trim($query));

        return Str::contains($key, $query);
    }

    /**
     * @param string $query the content of the query eg. 'AK'
     * @param string $locale the chosen local. Default to 'en'
     * @return Collection
     */
    public function findItem(string $query, string $locale = 'en'): Collection
    {
        return $this->items->filter(function ($val, $key) use ($query, $locale) {
            return $this->contains($val['_id'], $query)
                || $this->contains($val['_name'], $query)
                || $this->contains($val['_parent'], $query)
                || (($this->locales[$locale][$key]??false) ? ($this->contains($this->locales[$locale][$key]['Name'], $query) || $this->contains($this->locales[$locale][$key]['ShortName'], $query)) : false);
        })->map(function ($item) use ($locale) {
            return [
                '_id' => $item['_id'],
                '_name' => $item['_name'],
                'locale' => $this->locales[$locale][$item['_id']] ?? ''
            ];
        })->values();
    }

    /**
     * @param string $id the item ID to look for
     * @param string $locale the chosen local. Default to 'en'
     * @return array
     * @throws ItemNotFoundException
     */
    public function getItemById(string $id, string $locale = 'en'): array
    {
        $item =  $this->items[$id] ?? throw new ItemNotFoundException('Item not found');
        return [
            'item' => $item,
            'locale' => $this->locales[$locale][$id] ?? ''
        ];
    }
}
